SUBROUTINE IPTESF(FILEIN,FILEOUT,NFLDSKIP,LMASK,LXFLD,LYFLD,XMIN,XMAX,YMIN,YMAX, &
                  &IOUTFLD,FRMT,HEADING)
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
!
!   DUMPS SELECTED FIELDS IN BDB FILE TO AN ESF FILE
!
! KMF 8/12
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
USE WINTERACTER
USE INPPARAMS

IMPLICIT NONE
CHARACTER(LEN=255), INTENT(IN) :: FILEIN  !INTPUT FILE NAME
CHARACTER(LEN=255), INTENT(IN) :: FILEOUT !OUTPUT FILE NAME
INTEGER, INTENT(IN) :: NFLDSKIP           !FIELD TO LOOK FOR NULLS IN - IF 0 NO CHECKING DONE
LOGICAL, INTENT(IN) :: LMASK              !FLAG TO MASK OUTPUT
INTEGER, INTENT(IN) :: LXFLD              !FIELD FOR X IF MASKED OUTPUT
INTEGER, INTENT(IN) :: LYFLD              !FIELD FOR Y IF MASKED OUTPUT
REAL,  INTENT(IN) :: XMIN                 !X & Y LIMITS IF MASKED
REAL, INTENT(IN) :: XMAX
REAL, INTENT(IN) :: YMIN
REAL, INTENT(IN) :: YMAX
INTEGER, DIMENSION(MAXFLDS), INTENT(INOUT) :: IOUTFLD !ARRAY OF FLAGS TO OUTPUT OR NOT
CHARACTER(LEN=8),INTENT(INOUT) :: FRMT    !FORTRAN FORMAT SPECIFIER - 1 FORMAT FITS ALL
CHARACTER(LEN=80),INTENT(IN) :: HEADING   !DATA DESCRIPTOR FOR HEADER LINE 1

CHARACTER(LEN=500) :: OUTMESS
CHARACTER(LEN=2000) :: OUTSTR
CHARACTER(LEN=20), PARAMETER :: OUTSUB='IPTESF'
CHARACTER(LEN=30) :: TEMPSTR
INTEGER :: LUHIN
INTEGER :: MODE,I,IOUT,J
INTEGER :: ISTART,IEND
!INTEGER :: LENGTH                         !OUTPUT LENGTH OF SPECIFIED FORMAT
INTEGER :: IASTAT

CALL IUPPERCASE(FRMT)
ISTART=INDEX(FRMT,'G')
IF (ISTART == 0) THEN
  ISTART=INDEX(FRMT,'F')
END IF
IF (ISTART == 0) THEN
  OUTMESS='THERE IS NO G OR F IN YOUR FORMAT SPECIFIER. ONLY G OR F FORMATS ARE ALLOWED'
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
IEND=INDEX(FRMT,'.')
IF (IEND == 0) THEN
  OUTMESS='THERE IS NO DECIMAL POINT IN YOUR FORMAT SPECIFIER.'
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
!
!     WRITE ACTION TO PRT FILE
WRITE(LUHPRT,905)TRIM(FILEIN),TRIM(FILEOUT)
905 FORMAT(/,'CONVERTING DATABASE TO AN ESF FILE',/,'INPUT: ',A,/,'OUTPUT: ',A)
IF (LMASK) THEN
  IF (LXFLD < 1  .OR. LYFLD < 1) THEN
    OUTMESS='YOU HAVE SPECIFIED MASKING OF THE OUTPUT BUT NOT SET THE FIELD TO MASK WITH!'
    CALL WERRORBOX(OUTMESS,OUTSUB)
  END IF
  WRITE(LUHPRT,'("BASED ON THE FOLLOWING CONDITION:")')
  WRITE(LUHPRT,910) LXFLD,XMIN,XMAX
  WRITE(LUHPRT,910) LYFLD,YMIN,YMAX
  910 FORMAT(' FIELD ',I4,' MINIMUM =',G15.7,' MAXIMUM =',G15.7)
END IF
!
!     OPEN INPUT DATABASE AND GET HEADER
MODE=1
CALL BDBOPN(LUHIN,MODE,FILEIN)
CALL BDBHDR(LUHIN,MODE,NFLD,LOCX,LOCY,NPTS,LSORT,LABEL,ZMIN,ZMAX,LTYPE)
CALL BDBPRH(LUHIN,LUHPRT,FILEIN)
CALL BDBXHDR(LUHIN,MODE,HARR,HSTR)
!
!CHECK FOR EMPTY FILE
!
IF(NPTS .LE. 0) THEN
  OUTMESS='ERROR - BINARY DATABASE IS EMPTY '
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
!
!     CHECK CRITERIA FIELDS EXIST
!
IF (LMASK .AND. (LXFLD > NFLD .OR. LYFLD > NFLD)) THEN
  OUTMESS='YOU HAVE SPECIFIED A MASKING FIELD GREATER THAT THE NUMBER OF FIELDS IN THIS DATABASE'
  CALL WERRORBOX(OUTMESS,OUTSUB)
END IF
!
!GET WINDOW TIMES OR HARMONIC DATA
WTIME=0.0
NWIN=0
NHARM=0
DO I=1,NFLD-10
  IF (LABEL(I+10)(1:2).EQ.'CH') THEN
    IF (I > MAXTIMES) THEN
      WRITE(OUTMESS,'("Program is currently compiled for a maximum of ",i2," windows. &
      &This file has more - see Kim")') MAXTIMES
      CALL WERRORBOX(OUTMESS,OUTSUB)
    END IF
    READ(LABEL(I+10),'(5X,F7.0)',IOSTAT=IASTAT)WTIME(I)
    IF (IASTAT == 0) THEN
      NWIN=NWIN+1
    ELSE
      EXIT
    END IF
  ELSEIF (LABEL(I+10)(3:8) == '_MAGNT') THEN
      NHARM=NHARM+1
  ELSEIF (LABEL(I+10)(3:8) /= '_PHASE') THEN
    EXIT
  END IF
END DO
!
! REPLACE ANY SPACES IN THE LABELS
DO I=1,NFLD
  CALL LETSUB(' ','_',LABEL(I)(1:LEN_TRIM(LABEL(I))))
END DO  
!
!     OPEN OUTPUT FILE
CALL GETHANDL(LUHOUT)
OPEN(UNIT=LUHOUT,FILE=FILEOUT,ACCESS='SEQUENTIAL',FORM='FORMATTED',STATUS='UNKNOWN')
CALL ESFIPHEAD()
OUTSTR=''
!
!   GET STARTED
IOUT=0
CALL BDBRRB(-1,1,ZDATA,NFLD,NPTS)
DO I=1,NPTS
  !
  !   INPUT RECORD
  CALL BDBRRB(LUHIN,I,ZDATA,NFLD,NPTS)
  !
  !PRINT PROGRESS MESSAGE
  IF(MOD(I,1000) .EQ. 0) THEN
    CALL BDSTATUS(I,NPTS,ZNULL)
  END IF
  ! HAVE TO EMBED THE FOLLOWING TWO IF CHECKS TO AVOID REQUEST FOR ZDATA(0)
  ! TEST IF CHECKING FOR NULLS AND IF SO IS IT NULL?
  IF (NFLDSKIP >0 ) THEN
    IF (LNULL(ZDATA(NFLDSKIP))) CYCLE
  END IF
  !
  !TEST IF TRIMMING OUTPUT TO DEFINED AREA AND IF SO ARE WE IN THAT AREA?
  IF (LMASK) THEN
    IF((ZDATA(LXFLD).GE.XMAX).OR.(ZDATA(LXFLD).LE.XMIN).OR.(ZDATA(LYFLD).GE.YMAX).OR. &
          (ZDATA(LYFLD).LE.YMIN)) CYCLE
  END IF
  OUTSTR=''
  DO J=1,NFLD
    IF (IOUTFLD(J) > 0) THEN
      WRITE(TEMPSTR,FRMT)ZDATA(J)
      IF (IOUTFLD(J) == 30) THEN    !DATE YYMMDD
        IF (ZDATA(J) > 600000) THEN  !ASSUME NO IP DATA PRIOR TO 1960
          TEMPSTR=" 19"//ADJUSTL(TEMPSTR)
        ELSE
          TEMPSTR=" 20"//ADJUSTL(TEMPSTR)
        END IF
      END IF  
      OUTSTR=TRIM(OUTSTR)//TEMPSTR
    END IF  
  END DO
  IOUT=IOUT+1
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
END DO
IF (IOUT.EQ.0) THEN
  OUTMESS='NO POINTS FOUND WHICH FIT CRITERIA'
  CALL WINFOBOX(OUTMESS,OUTSUB)
END IF
WRITE(LUHPRT,'(/,"NUMBER OF POINTS OUTPUT: ",I10)')IOUT
!
!
CLOSE (LUHIN)
CLOSE (LUHOUT)
RETURN
CONTAINS
  SUBROUTINE ESFIPHEAD()
  !----------------------------------------------------------------------------------
  !
  ! PRINTS OUT EXTENDED HEADER AS A SERIES OF COMMENTS
  !
  !----------------------------------------------------------------------------------
  INTEGER :: I
  CHARACTER(LEN=200) :: TEMPSTR
  CHARACTER(LEN=18), DIMENSION(13), PARAMETER :: HEADSTR = (/'COMPANY          ','AREA             ', &
    &'PROJECT          ','CONTRACTOR       ','                 ','SURVEY_TYPE      ','TRANSMITTER      ',&
    &'RECEIVER         ','SENSOR           ','LINE_DIRECTION   ','UNITS.LENGTH     ','TRANSMITTER_CYCLE',&
    &'UNITS.EMIP       '/)
  CHARACTER(LEN=20), DIMENSION(18), PARAMETER :: HEADNUM = (/'GRADIENT_BLOCK_#    ', &
    &'TRANSMITTER_SERIAL_#','RECEIVER_SERIAL_#   ','TXFREQ              ',&
    &'MX_START            ','MX_END              ','                    ',&
    &'                    ','GRID.DECLINATION    ','MAG.DECLINATION     ',&
    &'DATE                ','DIPOLE              ','C1X                 ',&
    &'C1Y                 ','C1Z                 ','C2X                 ',&
    &'C2Y                 ','C2Z                 '/)
  CHARACTER(LEN=8), DIMENSION(34), PARAMETER :: ESFLABEL = (/'DUMMY   ','C1X     ', &
  &'C1Y     ','C1Z     ','C2X     ','C2Y     ','C2Z     ','P1X     ','P1Y     ',&
  &'P1Z     ','P2X     ','P2Y     ','P2Z     ','C1E_UTM ','C1N_UTM ','C2E_UTM ','C2N_UTM ',&
  &'P1E_UTM ','P1N_UTM ','P2E_UTM ','P2N_UTM ','C1D     ','C2D     ','P1D     ','P2D     ',&
  &'COLE_C  ','TAU     ','TXDIPOLE','NSTACK  ','DATE    ','RXDIPOLE','RS      ',&
  &'SD      ','SP      '/)
  
  !
  !   OUTPUT HEADER INFO
  WRITE(LUHOUT,'("VER:0001 ",A," ESF FILE EXTRACTED FROM BINARY FILE ",A)')TRIM(HEADING),TRIM(FILEIN)
  !CREATE RECORD TYPE 2
  WRITE(TEMPSTR,FMT=FRMT)ZNULL
  IF (NWIN > 0) THEN
    OUTSTR='DATATYPE:TDIP ARRAY:USER NULL:'
  ELSE
    OUTSTR='DATATYPE:FDIP ARRAY:USER NULL:'
  END IF    
  OUTSTR=TRIM(OUTSTR)//TRIM(ADJUSTL(TEMPSTR))
  DO I=1,13
    IF (LEN_TRIM(HSTR(I)) > 0 .AND. LEN_TRIM(HEADSTR(I)) > 0) THEN
      IF (INDEX(HSTR(I)(1:LEN_TRIM(HSTR(I))),' ') > 0 ) CALL LETSUB(' ','_',HSTR(I)(1:LEN_TRIM(HSTR(I))))
      IF (I == 9) THEN
        CALL IUPPERCASE(HSTR(I))
        IF (INDEX(HSTR(I),'STAINLESS') > 0 ) THEN
          HSTR(I)='STAINLESS'
        ELSEIF (INDEX(HSTR(I),'POROUS') > 0 ) THEN 
          HSTR(I)='POROUSPOT'
        END IF
      ELSEIF (I == 11)THEN
        CALL IUPPERCASE(HSTR(I))
        IF (INDEX(HSTR(I),'METRES') > 0 ) THEN
          HSTR(I)='M'
        ELSEIF (INDEX(HSTR(I),'FEET') > 0 ) THEN 
          HSTR(I)='FT'
        END IF
      END IF  
      WRITE(TEMPSTR,'(A,":",A)')TRIM(HEADSTR(I)),TRIM(HSTR(I))
      IF (LEN_TRIM(OUTSTR) > 0 ) THEN
        OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
      ELSE
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
      END IF      
      IF (LEN_TRIM(OUTSTR) > 80) THEN
        WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
        OUTSTR=''
      END IF
    END IF
  END DO
  DO I=1,18
    IF (.NOT. LNULL(HARR(I)) .AND. LEN_TRIM(HEADNUM(I)) > 0) THEN
      WRITE(TEMPSTR,'(A,":")')TRIM(HEADNUM(I))
      IF (LEN_TRIM(OUTSTR) > 0 ) THEN
        OUTSTR=TRIM(OUTSTR)//' '//TEMPSTR
      ELSE
        OUTSTR=TRIM(OUTSTR)//TEMPSTR
      END IF      
      WRITE(TEMPSTR,'(G15.7)')HARR(I)
      OUTSTR=TRIM(OUTSTR)//ADJUSTL(TEMPSTR)
      IF (LEN_TRIM(OUTSTR) > 80) THEN
        WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
        OUTSTR=''
      END IF
    END IF
  END DO
  IF (.NOT. LNULL(HARR(4))) THEN
    WRITE(TEMPSTR,'(F6.1)')1000.0/(HARR(4)*4.0)     !TIME IN MSEC
    OUTSTR=TRIM(OUTSTR)//" ONTIME:"//TRIM(ADJUSTL(TEMPSTR))//" OFFTIME:"//TRIM(ADJUSTL(TEMPSTR))
  END IF  
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  !CREATE RECORD TYPE 3
  IF (NWIN > 0) THEN
    OUTSTR='@DELAY='
    DO I=1,NWIN
      WRITE(TEMPSTR,'(F7.2)')WTIME(I)
      IF (I > 1) THEN
        OUTSTR=TRIM(OUTSTR)//','//TRIM(ADJUSTL(TEMPSTR))
      ELSE
        OUTSTR=TRIM(OUTSTR)//TRIM(ADJUSTL(TEMPSTR))
      END IF      
    END DO
  ELSE
    OUTSTR='@F='
    DO I=1,NHARM
      WRITE(TEMPSTR,'(F9.4)')HARR(4)*(I*2-1)
      IF (I > 1) THEN
        OUTSTR=TRIM(OUTSTR)//','//TRIM(ADJUSTL(TEMPSTR))
      ELSE
        OUTSTR=TRIM(OUTSTR)//TRIM(ADJUSTL(TEMPSTR))
      END IF      
    END DO
  END IF
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  IOUTFLD(1:10+MAX(NWIN,NHARM*2))=1
  !CREATE RECORD TYPE 4
  OUTSTR='LINE STATION'
  IF (.NOT. LNULL(ZMAX(3),ZMAX(4))) THEN
    OUTSTR=TRIM(OUTSTR)//' EAST NORTH'
    IF (.NOT. LNULL(ZMAX(5))) THEN
      OUTSTR=TRIM(OUTSTR)//' ELEVATION'
    ELSE
      IOUTFLD(5)=0
    END IF
  ELSE
    IOUTFLD(3)=0
    IOUTFLD(4)=0
  END IF
  OUTSTR=TRIM(OUTSTR)//' NSPACE CURRENT VP RES'
  IF (NWIN > 0) THEN
    OUTSTR=TRIM(OUTSTR)//' MX'
    DO I=1,NWIN
      OUTSTR=TRIM(OUTSTR)//' '//LABEL(10+I)(1:4)
    END DO
  ELSE
    OUTSTR=TRIM(OUTSTR)//' DECPH'
    DO I=1,NHARM
      WRITE(TEMPSTR,'("MAG",I1," PH",I1)')I,I
      OUTSTR=TRIM(OUTSTR)//' '//TRIM(TEMPSTR)
    END DO
  END IF
  DO I=11+MAX(NWIN,NHARM*2),NFLD
    IF (IOUTFLD(I) > 0) THEN
      IF (IOUTFLD(I) > 1) THEN 
        OUTSTR=TRIM(OUTSTR)//' '//ESFLABEL(IOUTFLD(I))
      ELSE
        OUTSTR=TRIM(OUTSTR)//' '//LABEL(I)
      END IF  
    END IF
  END DO
  WRITE(LUHOUT,'(A)')TRIM(OUTSTR)
  RETURN
  END SUBROUTINE ESFIPHEAD
END SUBROUTINE IPTESF
